classdef NvMAdmin < autosar.bsw.ServiceImplementation
% Class to define NvMAdmin

%   Copyright 2016 The MathWorks, Inc.
    
    properties(Constant, Access=private)
        DefaultClientPortName = 'NvMAdmin';
        DefaultInterfacePath = '/AUTOSAR/Services/NvM';
        InterfaceName = 'NvMAdmin';
        Operations = {'SetBlockProtection'};
        
        FunctionPrototypes = {'ERR = %s_SetBlockProtection(ProtectionEnabled)'};

        InputArgSpecs = {'boolean(true)'};
        
        OutputArgSpecs = {'uint8(1)'}; % P_Std_ReturnType
        
        FunctionPrototypeMap = containers.Map( autosar.bsw.NvMAdmin.Operations, ...
                                               autosar.bsw.NvMAdmin.FunctionPrototypes );
                                           
        InputArgSpecMap = containers.Map( autosar.bsw.NvMAdmin.Operations, ...
                                          autosar.bsw.NvMAdmin.InputArgSpecs );

        OutputArgSpecMap = containers.Map( autosar.bsw.NvMAdmin.Operations, ...
                                           autosar.bsw.NvMAdmin.OutputArgSpecs );
    end
    
    methods(Access=public)
        
        function this = NvMAdmin()
        end
        
        function updateFunctionCaller(this, blkPath, portName, operationName)

            functionPrototypeTemplate = this.FunctionPrototypeMap(operationName);
            inputArgSpec = this.InputArgSpecMap(operationName);
            outputArgSpec = this.OutputArgSpecMap(operationName);                        
            
            functionPrototype = sprintf(functionPrototypeTemplate, portName);
            
            set_param(blkPath, 'FunctionPrototype', functionPrototype);
            set_param(blkPath, 'InputArgumentSpecifications', inputArgSpec);
            set_param(blkPath, 'OutputArgumentSpecifications', outputArgSpec);
            
        end
    
    end
    
    methods(Static, Access=public)

        function deepCopyInterface(mdlName, dstInterfacePath) %#ok<INUSD>
        % Copy interface and data types to model
        
            arxmlPath = fullfile(autosarroot, '+autosar', '+bsw', '+arxml');
            arxmlFiles = {fullfile(arxmlPath, 'AUTOSAR_BaseTypes.arxml')
                          fullfile(arxmlPath, 'AUTOSAR_Type.arxml')
                          fullfile(arxmlPath, 'AUTOSAR_MOD_PlatformTypes.arxml')
                          fullfile(arxmlPath, 'AUTOSAR_NvM.arxml')};
            
            autosar.bsw.ServiceImplementation.updateReferences(mdlName, arxmlFiles);            
        end
        
        function defaultClientPortName = getDefaultClientPortName()
            defaultClientPortName =  autosar.bsw.NvMAdmin.DefaultClientPortName;
        end
       
        function interfaceName = getInterfaceName()
            interfaceName =  autosar.bsw.NvMAdmin.InterfaceName;
        end
        
        function operations = getOperations()
            operations =  autosar.bsw.NvMAdmin.Operations;
        end
        
        % This block does not expose a Datatype parameter because it does
        % not use any Enum types
        function hastypeParameter = hasDatatypeParameter()
            hastypeParameter = false;
        end
        
        function dataTypeCallback(~)
        end
        
        function operationCallback(~)
        end

        function desc = getDescription()
            newline = char(10);
            desc = ['Call an AUTOSAR NVRAM Manager (NvM) service function.', ...
                    newline, newline, ...
                    'Set the Client port name parameter to the port name used by the component for the function call.', ...
                    newline, newline, ...
                    'Select a NvM operation with the Operation parameter.  After the selection, ', ...
                    'the block inputs and outputs correspond to the input and output arguments of the selected operation.'];
        end
        
        function type = getType()
            type = 'NvMAdminCaller';
        end
        
        function help = getHelp()
            help = 'helpview(fullfile(docroot,''ecoder'',''helptargets.map''),''autosar_nvmadmin_block'');';
        end
        
    end
    
end

% LocalWords:  Nv bsw MAdmin helptargets nvmadmin
