classdef SimulinkFunction < handle
% Utility methods for Simulink Function and callers

%   Copyright 2016 The MathWorks, Inc.
    
    methods (Static, Access=public)

        function fcnHandles = getSimulinkFunctionsForCaller(callerH)
        % Lookup the many functions that the caller block might be calling (model needs
        % to be in a compiled state)
            fcnHandles = Simulink.AutosarTarget.ModelMapping.lookupSimulinkFunctions(callerH);
        end
        
        function isSubSystemServerRunnable = isGlobalSimulinkFunction(functionH)
        % Is the Simulink Function a global function?
            isSubSystemServerRunnable = Simulink.AutosarTarget.ModelMapping.isSubSystemServerRunnable(functionH);
        end

        function callerHandles = getGlobalFunctionCallerHandles(mdlName)
        % Get Global Function Callers
            mapping = autosar.api.Utils.modelMapping(mdlName);
            callerHandles = zeros(1, length(mapping.FunctionCallers));
            for callerIdx = 1:length(mapping.FunctionCallers)
                callerHandles(callerIdx) = get_param(mapping.FunctionCallers(callerIdx).Block, 'Handle');
            end
        end
        
        function sfcnHandles = getSFunctionHandles(sys)
        % Get SFunction handles for sys (model needs to be in a compiled state)
        
            % Find s-functions
            rt = sfroot;
            m = rt.find('-isa', 'Simulink.BlockDiagram', '-and', 'Name', sys);
            sfcns = m.find('-isa', 'Simulink.SFunction');
            
            sfcnHandles = zeros(1, length(sfcns));
            for sfcnIdx = 1:length(sfcns)
                sfcnHandles(sfcnIdx) = sfcns(sfcnIdx).handle;
            end
            
        end
        
        function str = removeTrailingSFunctionStr(str)
        % Remove special SFunction path string from path
            str = strrep(str, '/ SFunction ', '');
        end
        
    end
    
end

