classdef VariantValidator < autosar.validation.PhasedValidator
    %   Copyright 2015 The MathWorks, Inc.
    
    methods (Access=protected)
        function verifyInitial(this, hModel)
            assert(isscalar(hModel) && ishandle(hModel), 'hModel is not a handle');
            this.verifyGPCOn(hModel);
        end
    end
    
    methods (Static, Access=private)
        function verifyGPCOn(hModel)
            % Verify that all variant blocks have GPC property turned 'on'.
            % At time of writing, AUTOSAR does not support GPC off so we
            % throw a validation error if we find any blocks with that
            % parameter set.
            mdls = find_mdlrefs(hModel, 'Variants', 'AllVariants');
            for jj = length(mdls)
                variantBlocks = autosar.mm.util.getVariantBlocks(mdls(jj));
                for ii = 1:length(variantBlocks)
                    block = variantBlocks{ii};
                    gpc = get_param(block, 'GeneratePreprocessorConditionals');
                    if strcmp(gpc, 'off')
                        DAStudio.error(...
                            'autosarstandard:validation:VariantBlockGPCOffNotAllowed', ...
                            getfullname(block));
                    end
                end
            end
        end
    end
end